#!/bin/bash
#       
#   .
#
#  Copyright (C) 2015-2018 by Vladimir Oleynik <dzo@simtreas.ru>
#  may safely be consumed by a GPL license.


prg=`basename $0`
usage() {
	echo "Usage $prg [-D [-m]]" 1>&2
	echo " or" 1>&2
	echo "$prg" '-c [-1] [-m]|[-C]' 1>&2
	echo " or" 1>&2
	echo "$prg" '-C [-D]' 1>&2
	echo 1>&2
	echo " Options:" 1>&2
	echo "-D - destroy site outs" 1>&2
	echo "-m - make sitemap" 1>&2
	echo "-c - update index files" 1>&2
	echo "-1 - stop work after first update index file" 1>&2
	echo "-C - clear working outs" 1>&2
	exit 2
}

xc=
xC=
xD=
xm=
one=
while getopts ":cCmD1i:" opt; do
     case $opt in
	c)
	   [[ -n "$xD" ]] && usage
	   xc=c;;
	1) one=yes;;
	C)
	   [[ -n "$xm" ]] && usage
	   xC=C;;
	m)
	   [[ -n "$xC" ]] && usage
	   xm=m;;
	D)
	   [[ -n "$xc" ]] && usage
	   xD=D;;
	i) one=$OPTARG;;
	?) usage;;
     esac
done
if [[ $OPTIND = 1 ]]; then
	xm=m
	xc=c
else
	shift $((OPTIND-1))
fi

if [[ -n $one ]]; then
	[[ -z $xc ]] && usage
	[[ $one != yes ]] && cd `dirname $0`
fi

. ./var || exit 1

if [[ -z $out ]]; then
	echo Variable out do not set 1>&2
	exit 1
fi

if [[ -z $out_dirs_default_content ]]; then
	echo Variable out_dirs_default_content do not set 1>&2
	exit 1
fi

if [[ -n $ROOT ]]; then
	ROOT=${ROOT%/}
fi
findex=${findex:-index}
fmenus=${fmenus:-menu.html}
ftitles=${ftitles:-title.html}
faccordions=${faccordions:-accordions.html}
fnav=${fnav:-nav.html}
nav_sep=${nav_sep:-" / "}
fnav4=${fnav4:-nav4.html}
top_before_title=${top_before_title:-"top0*"}
style_add=${style_add:-"*.css"}
top_before_menu=${top_before_menu:-"top1*"}
top_before_accordion=${top_before_accordion:-"top2*"}
top_before_nav=${top_before_nav:-"top3*"}
top_before_h=${top_before_h:-"top4*"}
top_after_h=${top_after_h:-"top5*"}
top_before_nav4=${top_before_nav4:-"top6*"}
top_after_nav4=${top_after_nav4:-"top7*"}
top_before_content=${top_before_content:-"top8*"}
content=${content:-"content.*"}
top_after_content=${top_after_content:-"top9*"}
footer=${footer:-"zfooter.html"}
ext_link_add=${ext_link_add:-' &#8599;'}
prev_index_name=${prev_index_name:-".prev_index_name"}

make_name() {
	local __pth=$2 __mask=$3 __ff

	for __ff in $__pth/$__mask $__mask; do
		if [[ -f "$__ff" ]]; then
			eval $1=\$__ff
			return 0
		fi
	done
	if [[ -z $4 ]]; then
		echo "$__pth/$__mask and $__mask do not found" 1>&2
		exit 1
	fi
	return 1
}

out_first_or_add() {
	if (($1==0)); then
		echo "$3"
		echo "$2" > "$3"
	else
		echo "$2" >> "$3"
	fi
	(($1++))
}

make_a_href() {
	local __out=$1 __i=$2 __t=$3 __c=$4 __l=$5 __a

	if [[ 1 = $__c ]]; then             # "",1,0
		__c=" class=current"
	else
		__c=
	fi
	if [ -z "$__l" ]; then
		if [ "${DIM_TYPE[__i]}" ]; then
			if [[ ${DIM_TYPE[__i]} = ext ]]; then
				__l=${DIM_LINK[__i]}
			else
				__l=${ROOT}${DIM_LINK[__i]}
			fi
		else
			__l=${ROOT}${DIM_LINK[__i]}/
		fi
	fi
	if [[ ${DIM_TYPE[__i]} = ext ]]; then
		__a="<a$__c target=\"_blank\" href=\"$__l\">$__t$ext_link_add</a>"
	else
		__a="<a$__c href=\"$__l\">$__t</a>"
	fi
	eval $__out=\$__a
}

tst_regular_file() {
	if [ ! -f "$1" ]; then
		echo Error: "$1" is not regular file 1>&2
		exit 1
	fi
}

max_old=0

make_old_out() {
    local line l l2 n n2

    while IFS= read -r line ; do
	l=${line#DIM_LINK[}
	if [[ "$l" != "$line" ]]; then
		l2=${l#*]}
		n=${#l}
		n2=${#l2}
		n=${l:0:n-n2-1}
		if [[ n -gt 0 ]]; then
			IFS= read -r line
			l=${line#DIM_TYPE\[$n\]=}
			if [[ "$l" = "$line" || "$l" != ext ]]; then
				OLD_DIM_LINK[max_old++]=${l2:2:n2-3}
			fi
		fi
	fi
    done < "$1"
}

if [[ $OPTIND = 1 ]]; then

if [ -f "$out" ]; then
	make_old_out "$out"
fi

echo '# Automatic parsing, do not edit
' > "$out"

test_var_file() {
	local r
	while read r; do
	  if [ "$r" ]; then
		if [ "${r:0:1}" != '#' -a "${r#TITLE=}" = "$r" -a "${r#MENU=}" = "$r" -a "${r#HR=}" = "$r" -a "${r#LINK=}" = "$r" -a "${r#TYPE=}" = "$r" -a "${r#SCRIPT=}" = "$r" -a "${r#COPY=}" = "$r" -a "${r#COPY2=}" = "$r" ]; then
			echo "$1/var: can not parse $r line" 1>&2
			return 1
		fi
	  fi
	done < "$1"/var
}

reset_v() {
	MENU=
	TITLE=
	HR=
	LINK=
	TYPE=
	SCRIPT=
	COPY=
	COPY2=

	if [ ! -f "$1"/var ]; then
		echo "$1"/var not found 1>&2
		return 1
	fi

	test_var_file "$1" || exit 1

	# the file is tested, eval vars
	. "$1"/var || return 1

	if [ -z "$MENU" ]; then
		echo "$1"/var: have not set MENU, exiting 1>&2
		return 1
	fi
	HR=${HR:-"$MENU"}
	TITLE=${TITLE:-"$HR"}
	t=$TYPE
	if [[ -z $t || $t = ext || $t = nonav || $t = root || $t = sitemap || $t = skip ]]; then
		return 0
	else
		echo "$1/var: don know TYPE=$TYPE, exiting" 1>&2
		return 1
	fi
}

pth='..'
out_dims() {
	DIM_MENU[$line]=$MENU
	DIM_HR[$line]=$HR
	DIM_LINK[$line]=$LINK
	DIR_ID[$line]=$1

	[[ $TYPE != ext && $TYPE != skip ]] && echo "<title>$TITLE</title>" > "$1/$ftitles"
	{
	echo "DIM_MENU[$line]='$MENU'
DIM_HR[$line]='$HR'
DIR_ID[$line]='$1'
DIM_LINK[$line]='$LINK'"

	if [ "$TYPE" ]; then
		DIM_TYPE[$line]=$TYPE
		echo "DIM_TYPE[$line]=$TYPE"
	fi
	if [ "$SCRIPT" ]; then
		DIM_SCRIPT[$line]=$SCRIPT
		echo "DIM_SCRIPT[$line]='$SCRIPT'"
	fi
	if [ "$COPY" ]; then
		DIM_COPY[$line]=$COPY
		echo "DIM_COPY[$line]='$COPY'"
	fi
	if [ "$COPY2" ]; then
		DIM_COPY2[$line]=$COPY2
		echo "DIM_COPY2[$line]='$COPY2'"
	fi
	if [ "$2" ]; then
		PREV_LEVEL[$line]=$2
		echo "PREV_LEVEL[$line]=$2"
	fi

	echo
	} >> "$out"
	if [[ $TYPE != ext && $TYPE != skip ]]; then
		{
			echo '#!/bin/bash'
			echo "$pth/$prg -c -i $line"
		} > "$1/d_$prg"
		chmod +x "$1/d_$prg"
	fi
	((line++))
}

NNN_dir__2__dir() {
	local __out=$1 __f=$2 R="[0-9][0-9]*_(.*)"

	[[ "$__f" =~ $R ]]
	eval $__out='"${BASH_REMATCH[1]}"'
}

line=0
PREV_HR=
for l in [0-9]*_?* ; do

	[[ -d "$l" ]] || continue

	NNN_dir__2__dir fname "$l"

	if [ "$fname" ]; then
		echo $fname

		reset_v "$l" || exit 1

		if [[ -z $LINK ]]; then
			if [[ line -ne 0 ]]; then
				LINK="/$fname"
			fi
		fi

		out_dims "$l" ""
	fi
done

menu_1_max=$((line-1))
echo "menu_1_max=$menu_1_max
" >> "$out"


echo "
Next level
"
pth='../..'
PREV_LINK=
for (( i=0; i<=menu_1_max; i++ )); do
	# load next level

	[[ i -ne 0 ]] && PREV_LINK=${DIM_LINK[i]}

	PREV_HR=${DIM_HR[i]}

	for l in ${DIR_ID[i]}/[0-9]*_?* ; do

	  [[ -d "$l" ]] || continue

	  # "level1/NNN_dir" to "dir"
	  NNN_dir__2__dir fname "${l#${DIR_ID[i]}/}"

	  if [ "$fname" ]; then
		  echo $fname

		  reset_v "$l" || exit 1

		  if [ -z "$LINK" ]; then
			if [ "$PREV_LINK" ]; then
				LINK="$PREV_LINK/$fname"
			else
				make_name fcontent "$l" "$content"
				ext=${fcontent##*.}
				if [ x"$ext" = x"$fcontent" ]; then
					echo .ext from "$fcontent" do not found, exiting 1>&2
					exit 1
				fi
				LINK="/$fname"."$ext"
				TYPE=${TYPE:-root}
			fi
		  fi

		  out_dims "$l" $i
	  fi
	done
done
menu_2_max=$((line-1))
echo "menu_2_max=$menu_2_max
" >> "$out"


echo "
Make top menu
"
make_top_menu_level1() {
  current=$1
  index=$2

  echo ' <!-- start main menu -->'

  # skip /index
  for (( i=1; i<=menu_1_max; i++ )); do
	t=${DIM_TYPE[i]}
	[[ $t = skip || $t = nonav ]] && continue

	if [ x${DIM_LINK[i]} = x"$current" ]; then
		if [[ index -eq 1 ]]; then
			make_a_href m1 $i "${DIM_MENU[i]}" 1
		else
			make_a_href m1 $i "${DIM_MENU[i]}" 1 "#"
		fi
	else
		make_a_href m1 $i "${DIM_MENU[i]}"
	fi
	if [[ i -eq menu_1_max ]]; then
		echo -n " <li class='right'>$m1"
	else
		echo -n " <li>$m1"
	fi

	ul=0
	for (( i2=menu_1_max+1; i2<=menu_2_max; i2++ )); do
		t=${DIM_TYPE[i2]}
		[[ $t = skip || $t = nonav ]] && continue

		if [[ PREV_LEVEL[i2] -eq i ]]; then
		 if [[ ul -eq 0 ]]; then
			ul=1
			echo '<ul>'
		 fi
		 make_a_href m2 $i2 "${DIM_MENU[i2]}"
		 echo "   <li>$m2</li>"
		fi
	done
	if [[ ul -ne 0 ]]; then
		echo $'  </ul>\n </li>'
	else
		echo '</li>'
	fi
  done
  echo ' <!-- end main menu -->'
}

echo "${DIR_ID[0]}/$fmenus"
make_top_menu_level1  ${DIM_LINK[0]} 1 > "${DIR_ID[0]}/$fmenus"
for (( l=1; l<=menu_1_max; l++ )); do
	t=${DIM_TYPE[l]}
	[[ $t = skip || $t = ext ]] && continue

	echo "${DIR_ID[l]}/$fmenus"
	make_top_menu_level1 ${DIM_LINK[l]} 0 > "${DIR_ID[l]}/$fmenus"
done

make_top_menu_level2() {
  current1=$1
  current2=$2

  echo ' <!-- start main menu -->'
  # skip /index
  for (( i=1; i<=menu_1_max; i++ )); do
	t=${DIM_TYPE[i]}
	[[ $t = skip || $t = nonav ]] && continue

	if [ x${DIM_LINK[i]} = x"$current1" ]; then
		cu=1
	else
		cu=0
	fi
	make_a_href m1 $i "${DIM_MENU[i]}" $cu
	if [[ i -eq menu_1_max ]]; then
		echo -n " <li class='right'>$m1"
	else
		echo -n " <li>$m1"
	fi

	ul=0
	for (( i2=menu_1_max+1; i2<=menu_2_max; i2++ )); do
		t=${DIM_TYPE[i2]}
		[[ $t = skip || $t = nonav ]] && continue

		if [[ PREV_LEVEL[i2] -eq i ]]; then
		 if [[ ul -eq 0 ]]; then
			ul=1
			echo '<ul>'
		 fi
		 if [ x${DIM_LINK[i2]} = x"$current2" ]; then
			echo "   <li class=current>${DIM_MENU[i2]}</li>"
		 else
			make_a_href m2 $i2 "${DIM_MENU[i2]}"
			echo "   <li>$m2</li>"
		 fi
		fi
	done
	if [[ ul -ne 0 ]]; then
		echo $'  </ul>\n </li>'
	else
		echo '</li>'
	fi
  done
  echo ' <!-- end main menu -->'
}

for (( l=menu_1_max+1; l<=menu_2_max; l++ )); do
	t=${DIM_TYPE[l]}
	[[ $t = skip || $t = ext ]] && continue

	pl=${PREV_LEVEL[l]}
	if [[ pl -eq 0 ]]; then
	  cl=/
	else
	  cl=${DIM_LINK[pl]}
	fi
	echo "${DIR_ID[l]}/$fmenus"
	make_top_menu_level2 "$cl" "${DIM_LINK[l]}" > "${DIR_ID[l]}/$fmenus"
done

scan_level_2_4() {
    echo "
Next level
"
    local max=$line
    for (( i=$1+1; i<max; i++ )); do
	# load next level

	PREV_LINK=${DIM_LINK[i]}
	PREV_HR=${DIM_HR[i]}

	for l in ${DIR_ID[i]}/[0-9]*_?* ; do

	  [[ -d "$l" ]] || continue

	  # "level1/*/NNN_dir" to "dir"
	  NNN_dir__2__dir fname "${l#${DIR_ID[i]}/}"

	  if [ "$fname" ]; then
		  echo $fname

		  reset_v "$l" || exit 1

		  LINK=${LINK:-"$PREV_LINK/$fname"}
		  if [ -n "$2" -a "$TYPE" = skip ]; then
			echo "WARINIG: $l ignored - it last level" 1>&2
		  else
			if [ "$2" ]; then
				TYPE=${TYPE:-nonav}
			fi
			out_dims "$l" $i
		  fi
	  fi
	done
    done
}

line=$((menu_2_max+1))

pth='../../..'
scan_level_2_4 $menu_1_max

menu_3_max=$((line-1))
echo "menu_3_max=$menu_3_max
" >> "$out"

make_top_menu_level_more2() {
  current1=$1
  current2=$2

  echo ' <!-- start main menu -->'
  # skip /index
  for (( i=1; i<=menu_1_max; i++ )); do
	t=${DIM_TYPE[i]}
	[[ $t = skip || $t = nonav ]] && continue

	if [ x"${DIM_LINK[i]}" = x"$current1" ]; then
		cu=1
	else
		cu=0
	fi
	make_a_href m1 $i "${DIM_MENU[i]}" $cu
	if [[ i -eq menu_1_max ]]; then
		echo -n " <li class='right'>$m1"
	else
		echo -n " <li>$m1"
	fi

	ul=0
	for (( i2=menu_1_max+1; i2<=menu_2_max; i2++ )); do
		t=${DIM_TYPE[i2]}
		[[ $t = skip || $t = nonav ]] && continue

		if [[ PREV_LEVEL[i2] -eq i ]]; then
		 if [[ ul -eq 0 ]]; then
			ul=1
			echo '<ul>'
		 fi
		 if [ x"${DIM_LINK[i2]}" = x"$current2" ]; then
			cu2=1
		 else
			cu2=0
		 fi
		 make_a_href m2 $i2 "${DIM_MENU[i2]}" $cu2
		 echo "   <li>$m2</li>"
		fi
	done
	if [[ ul -ne 0 ]]; then
		echo $'  </ul>\n </li>'
	else
		echo '</li>'
	fi
  done
  echo ' <!-- end main menu -->'
}

if [[ menu_3_max -eq menu_2_max ]]; then
	menu_4_max=$menu_3_max
	menu_5_max=$menu_3_max
	echo menu_4_max=$menu_4_max >> "$out"
	echo menu_5_max=$menu_5_max >> "$out"
	echo
	echo third level is empty
	echo
else

echo "
Make menu for third level
"
for (( l=menu_2_max+1; l<=menu_3_max; l++ )); do
	t=${DIM_TYPE[l]}
	[[ $t = skip || $t = ext ]] && continue

	pl2=${PREV_LEVEL[l]}
	pl1=${PREV_LEVEL[pl2]}
	if [[ pl1 -gt 0 ]]; then
	    echo "${DIR_ID[l]}/$fmenus"
	    make_top_menu_level_more2 "${DIM_LINK[pl1]}" "${DIM_LINK[pl2]}" > "${DIR_ID[l]}/$fmenus"
	fi
done

line=$((menu_3_max+1))

pth='../../../..'
scan_level_2_4 $menu_2_max

menu_4_max=$((line-1))
echo "menu_4_max=$menu_4_max
" >> "$out"

if [[ menu_4_max -eq menu_3_max ]]; then
	menu_5_max=$menu_4_max
	echo menu_5_max=$menu_5_max >> "$out"
	echo
	echo 4-th level is empty
	echo
else
	echo
	echo Make menu for 4-th level
	echo
	for (( l=menu_3_max+1; l<=menu_4_max; l++ )); do
		t=${DIM_TYPE[l]}
		[[ $t = skip || $t = ext ]] && continue

		pl=${PREV_LEVEL[l]}
		pl2=${PREV_LEVEL[pl]}
		pl1=${PREV_LEVEL[pl2]}
		if [[ pl1 -gt 0 ]]; then
		    echo "${DIR_ID[l]}/$fmenus"
		    make_top_menu_level_more2 ${DIM_LINK[pl1]} ${DIM_LINK[pl2]} > "${DIR_ID[l]}/$fmenus"
		fi
	done
fi


line=$((menu_4_max+1))

pth='../../../../..'
scan_level_2_4 $menu_3_max last

menu_5_max=$((line-1))
echo "menu_5_max=$menu_5_max
" >> "$out"

if [[ menu_5_max -eq menu_4_max ]]; then
	echo
	echo 5-th level is empty
	echo
else
	echo
	echo Copy menu for 5-th level from 4-th level
	echo
	for (( l=menu_4_max+1; l<=menu_5_max; l++ )); do
		if [[ ext != ${DIM_TYPE[l]} ]]; then
		  echo "${DIR_ID[l]}/$fmenus"
		  pl=${PREV_LEVEL[l]}
		  while [ skip = "${DIM_TYPE[pl]}" ]; do
			pl=${PREV_LEVEL[pl]}
		  done
		  cp "${DIR_ID[pl]}/$fmenus" "${DIR_ID[l]}" || exit 1
		fi
	done
fi # 5-th level is not empty

fi      # third level is not empty


echo "
Make navigation
"
# skip index
for (( i=1; i<=menu_5_max; i++ )); do
	[[ ${DIM_TYPE[i]} = ext || ${DIM_TYPE[i]} = skip ]] && continue

	NAV=
	pl=$i
	while [[ pl -gt 0 ]]; do
		pl=${PREV_LEVEL[pl]}
		pl=${pl:-0}
		if [[ skip != ${DIM_TYPE[pl]} ]]; then
			if [[ pl -eq 0 ]]; then
				make_a_href l $pl "${DIM_HR[pl]}" "" "$ROOT/"
			else
				make_a_href l $pl "${DIM_HR[pl]}"
			fi
			if [ "$NAV" ]; then
				NAV="$l$nav_sep$NAV"
			else
				NAV=$l
			fi
		fi
	done
	echo "$NAV" > "${DIR_ID[i]}/$fnav"
done


echo "
Make 4-th level navigation
"

for (( i=menu_2_max+1; i<=menu_3_max; i++ )); do
	[[ ${DIM_TYPE[i]} = ext ]] && continue

	f=${DIR_ID[$i]}/$fnav4
	n=0
	for (( l=menu_3_max+1; l<=menu_4_max; l++ )); do
		t=${DIM_TYPE[l]}
		[[ $t = skip || $t = nonav ]] && continue

		if [[ PREV_LEVEL[l] -eq i ]]; then
			make_a_href m $l "${DIM_MENU[l]}"
			out_first_or_add n "   <li>$m</li>" "$f"
		fi
	done
done

for (( i=menu_3_max+1; i<=menu_4_max; i++ )); do
	t=${DIM_TYPE[i]}
	[[ $t = skip || $t = nonav || $t = ext ]] && continue

	n=0
	f=${DIR_ID[$i]}/$fnav4

	for (( l=menu_3_max+1; l<=menu_4_max; l++ )); do
	  t=${DIM_TYPE[l]}
	  [[ $t = skip || $t = nonav ]] && continue

	  if [[ PREV_LEVEL[l] -eq PREV_LEVEL[i] ]]; then
		if [ x${DIM_LINK[i]} = x${DIM_LINK[l]} ]; then
			out_first_or_add n "   <li class=current>${DIM_MENU[l]}</li>" "$f"
		else
			make_a_href m $l "${DIM_MENU[l]}"
			out_first_or_add n "   <li>$m</li>" "$f"
		fi
	  fi
	done
done


echo "
Make accordions
"

# skip make accordion for /index
for (( i=1; i<=menu_1_max; i++ )); do
  t=${DIM_TYPE[i]}
  [[ $t = skip || $t = ext ]] && continue

  a=${DIR_ID[i]}/$faccordions
  echo $a
  for (( l=menu_1_max+1; l<=menu_2_max; l++ )); do
   if [[ PREV_LEVEL[l] -eq i ]]; then
    t=${DIM_TYPE[l]}
    [[ $t = skip || $t = nonav ]] && continue

    make_a_href am1 $l "${DIM_MENU[l]}"
    echo " <li class="left_menu_item">$am1</li>"
   fi
  done > "$a"
done

echo

for (( a2=menu_1_max+1; a2<=menu_2_max; a2++ )); do
  t=${DIM_TYPE[a2]}
  [[ $t = skip || $t = ext ]] && continue

  a=${DIR_ID[a2]}/$faccordions
  echo $a
  for (( i=0; i<=menu_1_max; i++ )); do

    if [[ PREV_LEVEL[a2] -eq i ]]; then
	t=${DIM_TYPE[i]}
	[[ $t = skip || $t = nonav ]] && continue

	for (( l=menu_1_max+1; l<=menu_2_max; l++ )); do
	 if [[ PREV_LEVEL[l] -eq PREV_LEVEL[a2] ]]; then
	  t=${DIM_TYPE[l]}
	  [[ $t = skip || $t = nonav ]] && continue

	  if [ x${DIM_LINK[l]} = x${DIM_LINK[a2]} ]; then
	   echo -n "  <li class='left_menu_item current'>${DIM_MENU[l]}"
	   ul=0
	   for (( l2=menu_2_max+1; l2<=menu_3_max; l2++ )); do
	    t=${DIM_TYPE[l2]}
	    [[ $t = skip || $t = nonav ]] && continue

	    if [[ PREV_LEVEL[l2] -eq l ]]; then
		 if [[ ul -eq 0 ]]; then
			ul=1
			echo '<ul class="left_menu2">'
		 fi
		 make_a_href am2 $l2 "${DIM_MENU[l2]}"
		 echo "    <li>$am2</li>"
	    fi
	   done
	   if [[ ul -eq 1 ]]; then
		echo $'   </ul>\n  </li>'
	   else
		echo '</li>'
	   fi
	  else
	   make_a_href am1 $l "${DIM_MENU[l]}"
	   echo "  <li class='left_menu_item'>$am1</li>"
	  fi
	 fi
	done
    fi
  done > "$a"
done

echo

for (( a3=menu_2_max+1; a3<=menu_3_max; a3++ )); do
  t=${DIM_TYPE[a3]}
  [[ $t = skip || $t = ext ]] && continue

  a=${DIR_ID[a3]}/$faccordions
  echo $a
  pa=${PREV_LEVEL[a3]}
  for (( i=0; i<=menu_1_max; i++ )); do
    if [[ PREV_LEVEL[pa] -eq i ]]; then
	t=${DIM_TYPE[i]}
	[[ $t = skip || $t = nonav ]] && continue

	for (( l=menu_1_max+1; l<=menu_2_max; l++ )); do
	 if [[ PREV_LEVEL[l] -eq PREV_LEVEL[pa] ]]; then
	  t=${DIM_TYPE[l]}
	  [[ $t = skip || $t = nonav ]] && continue

	  make_a_href am1 $l "${DIM_MENU[l]}"
	  if [ x${DIM_LINK[l]} = x${DIM_LINK[pa]} ]; then
	   echo -n "  <li class='left_menu_item current'>$am1"
	   ul=0
	   for (( l2=menu_2_max+1; l2<=menu_3_max; l2++ )); do
	    if [[ PREV_LEVEL[l2] -eq l ]]; then
		 t=${DIM_TYPE[l2]}
		 [[ $t = skip || $t = nonav ]] && continue

		 if [[ ul -eq 0 ]]; then
			ul=1
			echo '<ul class="left_menu2">'
		 fi
		 if [ x${DIM_LINK[l2]} = x${DIM_LINK[a3]} ]; then
		   echo "    <li class=current>${DIM_MENU[l2]}</li>"
		 else
		   make_a_href am2 $l2 "${DIM_MENU[l2]}"
		   echo "    <li>$am2</li>"
		 fi
	    fi
	   done
	   if [[ ul -eq 1 ]]; then
		echo $'   </ul>\n  </li>'
	   else
		echo '</li>'
	   fi
	  else
	   echo "  <li class='left_menu_item'>$am1</li>"
	  fi

	 fi
	done
    fi
  done > "$a"
done

echo

for (( a4=menu_3_max+1; a4<=menu_4_max; a4++ )); do
  t=${DIM_TYPE[a4]}
  [[ $t = skip || $t = ext ]] && continue

  a=${DIR_ID[a4]}/$faccordions
  echo $a
  pa=${PREV_LEVEL[a4]}
  pa=${PREV_LEVEL[pa]}
  for (( i=0; i<=menu_1_max; i++ )); do
    if [[ PREV_LEVEL[pa] -eq i ]]; then
	t=${DIM_TYPE[i]}
	[[ $t = skip || $t = nonav ]] && continue

	for (( l=menu_1_max+1; l<=menu_2_max; l++ )); do
	 if [[ PREV_LEVEL[l] -eq PREV_LEVEL[pa] ]]; then
	  t=${DIM_TYPE[l]}
	  [[ $t = skip || $t = nonav ]] && continue

	  make_a_href am1 $l "${DIM_MENU[l]}"
	  if [ x${DIM_LINK[l]} = x${DIM_LINK[pa]} ]; then
	   echo -n "  <li class='left_menu_item current'>$am1"
	   ul=0
	   for (( l2=menu_2_max+1; l2<=menu_3_max; l2++ )); do
	    if [[ PREV_LEVEL[l2] -eq l ]]; then
		 t=${DIM_TYPE[l2]}
		 [[ $t = skip || $t = nonav ]] && continue

		 if [[ ul -eq 0 ]]; then
			ul=1
			echo '<ul class="left_menu2">'
		 fi
		 make_a_href am2 $l2 "${DIM_MENU[l2]}"
		 if [[ PREV_LEVEL[a4] -eq l2 ]]; then
			echo "    <li class=current>$am2</li>"
		 else
			echo "    <li>$am2</li>"
		 fi
	    fi
	   done
	   if [[ ul -eq 1 ]]; then
		echo $'   </ul>\n  </li>'
	   else
		echo '</li>'
	   fi
	  else
	   echo "  <li class='left_menu_item'>$am1</li>"
	  fi

	 fi
	done
    fi
  done > "$a"
done

for (( l=menu_4_max+1; l<=menu_5_max; l++ )); do
	if [[ ext != ${DIM_TYPE[l]} ]]; then
		echo "${DIR_ID[l]}/$faccordions"
		pl=${PREV_LEVEL[l]}
		while [ "${DIM_TYPE[pl]}" = skip ]; do
			pl=${PREV_LEVEL[pl]}
		done
		cp "${DIR_ID[pl]}/$faccordions" "${DIR_ID[l]}" || exit 1
	fi
done

# end call without opt
else
	if [ ! -f ./"$out" ]; then
		echo "$out not found, use $0 without option" 1>&2
		exit 1
	fi
	. ./"$out"
	if [ -z "$menu_5_max" ]; then
		echo "$out incomplete, use $0 without option" 1>&2
		exit 1
	fi
fi


if [[ $xm = m ]]; then

echo "
Make sitemap
"
for (( i=menu_1_max+1; i<=menu_2_max; i++ )); do
	[[ sitemap = ${DIM_TYPE[i]} ]] && break
done
if [[ i -gt menu_2_max ]]; then
	echo TYPE=sitemap in first level submenu not found, exiting 1>&2
	exit 1
fi
if [[ PREV_LEVEL[i] -ne 0 ]]; then
	echo TYPE=sitemap must set in index first level menu, exiting 1>&2
	exit 1
fi
f=${content%.*}
f=${DIR_ID[i]}/${f}.html
{
echo '<ul id=sitemap>'
li1=0
li2=0
li3=0
for (( i=0; i<=menu_1_max; i++ )); do
  t1=x${DIM_TYPE[i]}
  [[ $t1 = xnonav ]] && continue

  if [ $t1 != xskip ]; then
	if [[ i -eq 0 ]]; then
		make_a_href m1 0 "${DIM_MENU[0]}" "" "$ROOT/"
	else
		make_a_href m1 $i "${DIM_MENU[i]}"
	fi
	echo -n " <li>$m1"
	li1=1
  fi
  ul2=0
  for (( i2=menu_1_max+1; i2<=menu_2_max; i2++ )); do
      [[ PREV_LEVEL[i2] -ne i ]] && continue

      t2=x${DIM_TYPE[i2]}
      [[ $t2 = xnonav ]] && continue

      if [ $t2 != xskip ]; then
	if [ $ul2 -eq 0 -a ! $t1 = xskip ]; then
	  ul2=1
	  echo '<ul>'
	fi
	if [ $t2 = xsitemap ]; then
		make_a_href m2 $i2 "${DIM_MENU[i2]}" 1 '#'
	else
		make_a_href m2 $i2 "${DIM_MENU[i2]}"
	fi
	echo -n "  <li>$m2"
	li2=1
      fi
      ul3=0
      for (( i3=menu_2_max+1; i3<=menu_3_max; i3++ )); do
	[[ PREV_LEVEL[i3] -ne i2 ]] && continue

	t3=x${DIM_TYPE[i3]}
	[[ $t3 = xnonav ]] && continue

	if [ $t3 != xskip ]; then
		if [ $ul3 -eq 0 -a ! $t2 = xskip ]; then
			ul3=1
			echo '<ul>'
		fi
		make_a_href m3 $i3 "${DIM_MENU[i3]}"
		echo -n "   <li>$m3"
		li3=1
	fi
	ul4=0
	for (( i4=menu_3_max+1; i4<=menu_4_max; i4++ )); do
	  [[ PREV_LEVEL[i4] -ne i3 ]] && continue

	  t4=x${DIM_TYPE[i4]}
	  [[ $t4 = xnonav ]] && continue

	  if [ $t4 != xskip ]; then
		if [ $ul4 -eq 0 -a ! $t3 = xskip ]; then
			ul4=1
			echo '<ul>'
		fi
		make_a_href m4 $i4 "${DIM_MENU[i4]}"
		echo "    <li>$m4</li>"
	  fi
	done
	if [[ ul4 -ne 0 ]]; then
	  echo '   </ul>'
	fi
	if [[ li3 -ne 0 ]]; then
		if [[ ul4 -ne 0 ]]; then
			echo -n '   '
		fi
		echo '</li>'
		li3=0
	fi
      done
      if [[ ul3 -ne 0 ]]; then
	  echo '  </ul>'
      fi
      if [[ li2 -ne 0 ]]; then
	if [[ ul3 -ne 0 ]]; then
		echo -n '  '
	fi
	echo '</li>'
	li2=0
      fi
  done
  if [[ ul2 -ne 0 ]]; then
      echo ' </ul>'
  fi
  if [[ li1 -ne 0 ]]; then
	if [[ ul2 -ne 0 ]]; then
		echo -n ' '
	fi
	echo '</li>'
	li1=0
  fi
done
echo '</ul>'
} > "$f"
# end without opt or -m
fi

read_file_into_var() {
	local __out_var_name=$1 __var_name=$2
	local __mask=${!__var_name}
	local __FILES_LIST=($__mask)
	local __o __line

	if [ ${#__FILES_LIST[@]} -eq 0 ]; then
		echo Error: empty mask "$__var_name" 1>&2
		exit 1
	fi
	if [ ${#__FILES_LIST[@]} -gt 1 ]; then
		echo Error: mask "$__var_name=$__mask" match more one files 1>&2
		exit 1
	fi
	if [ ! -e "${__FILES_LIST[0]}" ]; then
		if [ -z "$3" ]; then
			echo Error: "$__var_name=$__mask" is not exist 1>&2
			exit 1
		fi
		return 1
	fi
	tst_regular_file "${__FILES_LIST[0]}"

	while IFS= read -r __line ; do
		if [ -z "$__o" ]; then
			__o=$__line
		else
			__o=$__o$'\n'$__line
		fi
	done < "${__FILES_LIST[0]}"
	eval $__out_var_name=\$__o
}

if [[ $xc = c ]]; then
echo "
Make indexes
"


read_file_into_var top0 top_before_title
style2=
read_file_into_var style2 style_add no_error
read_file_into_var top1 top_before_menu
read_file_into_var top2 top_before_accordion
read_file_into_var top3 top_before_nav
read_file_into_var top4 top_before_h
read_file_into_var top5 top_after_h
read_file_into_var top6 top_before_nav4
read_file_into_var top7 top_after_nav4
read_file_into_var top8 top_before_content
read_file_into_var top9 top_after_content
read_file_into_var zfooter footer

put_file_to_out() {
	local optn=$1
	if [ x"$optn" = x-n ]; then
		shift
	else
		optn=
	fi
	local mask=$1
	local FILES_LIST=($mask)
	local f

	if [ ${#FILES_LIST[@]} -gt 1 ]; then
		echo Error: mask "$mask" match more one files 1>&2
		exit 1
	fi
	f=${FILES_LIST[0]}
	if [ ! -e "$f" ]; then
		if [ -z "$2" ]; then
			echo Error: "$mask" is not exist 1>&2
			exit 1
		fi
		if [ -n "${!2}" ]; then
			if [ -z "$optn" ]; then
				echo "${!2}"
			else
				echo -n "${!2}"
			fi
		fi
		return 0
	fi
	tst_regular_file "$f"

	if [[ -z $optn ]]; then
		cat "$f"
	else
		mask=0
		while IFS= read -r optn ; do
			if [[ mask -ne 0 ]]; then
				echo
			else
				mask=1
			fi
			echo -n "$optn"
		done < "$f"
	fi
}


u=0
nu=0
nc=0
for (( i=0; i<=menu_5_max; i++ )); do
  if [[ $one =~ [0-9][0-9]* ]]; then
	[[ one -ne i ]] && continue
  fi
  t=${DIM_TYPE[i]}
  [[ $t = ext ]] && continue

  if [[ $t = skip ]]; then
	echo "..${DIM_LINK[i]}"
	pl=${PREV_LEVEL[i]} # pl={"",0,1...}
	if [[ $pl != 0 ]]; then
		  mkdir -p ..${DIM_LINK[i]}
	fi
	while [ "${DIM_TYPE[pl]}" = skip ]; do
		pl=${PREV_LEVEL[pl]}
	done
	if [[ -z $pl ]]; then
		index="../$findex.*"
	else if [[ pl -eq 0 ]]; then
		index="..${DIM_LINK[pl]}"
	     else
		index="..${DIM_LINK[pl]}/$findex.*"
	     fi
	fi
	cp $index "..${DIM_LINK[i]}" || exit 1
	continue
  fi

  make_name fcontent "${DIR_ID[i]}" "$content"
  ext=${fcontent##*.}
  if [ x"$ext" = x"$fcontent" ]; then
	  echo .ext from "$fcontent" do not found, exiting 1>&2
	  exit 1
  fi

  if [ "${fcontent#*/}" = "$fcontent" ]; then
	  out_first_or_add nc "${DIR_ID[i]}" "$out_dirs_default_content.$$"
  fi

  if [[ i -eq 0 ]]; then
	  d=..
	  index="../$findex.$ext"
  else
	  d=..${DIM_LINK[i]}
	  if [[ ${PREV_LEVEL[i]} = 0 ]]; then   # pl={"",0,1...}
		  index=$d
	  else
		  mkdir -p "$d"
		  index="$d/$findex.$ext"
	  fi
  fi
  echo -n "$index "

  if [ "${DIM_SCRIPT[i]}" ]; then
	"${DIR_ID[i]}/"${DIM_SCRIPT[i]} "$fcontent" "$index"
  fi

  if [ "${DIM_COPY[i]}" ]; then
	pushd "${DIR_ID[i]}" > /dev/null
	FILES_LIST=(${DIM_COPY[i]})
	popd > /dev/null
	t=0
	for l in ${FILES_LIST[@]}; do
		r=$d/$l
		FILES_LIST[t]=${DIR_ID[i]}/$l
		l=${FILES_LIST[t]}
		if [ ! -e "$l" ]; then
			echo Error: "$l" is not exist 1>&2
			exit 1
		fi
		tst_regular_file "$l"
		[[ ! "$l" -nt "$r" ]] && unset FILES_LIST[t]
		((t++))
	done
	[[ ${#FILES_LIST[@]} -ne 0 ]] && cp -a ${FILES_LIST[@]} "$d"
  fi

  if [ "${DIM_COPY2[i]}" ]; then
	FILES_LIST=(${DIM_COPY2[i]})
	for (( t=0; t<${#FILES_LIST[@]}; t++ )); do
		l=${DIR_ID[i]}/${FILES_LIST[t]}
		if [ ! -e "$l" ]; then
			echo Error: "$l" is not exist 1>&2
			exit 1
		fi
		tst_regular_file "$l"
		((t++))
		if [ ${#FILES_LIST[@]} -eq $t ]; then
			echo Error: COPY2 must have pairs 1>&2
			exit 1
		fi
		r=$d/${FILES_LIST[t]}
		[[ "$l" -nt "$r" ]] && cp -a "$l" "$r"
	done
  fi

  index_tmp=$index.$$

  {
  put_file_to_out "${DIR_ID[i]}/$top_before_title" top0

  put_file_to_out "${DIR_ID[i]}/$ftitles"

  put_file_to_out "${DIR_ID[i]}/$style_add" style2

  put_file_to_out "${DIR_ID[i]}/$top_before_menu" top1

  put_file_to_out "${DIR_ID[i]}/$fmenus"

  if [[ i -ne 0 ]]; then
	  put_file_to_out "${DIR_ID[i]}/$top_before_accordion" top2

	  put_file_to_out "${DIR_ID[i]}/$faccordions"

	  # -n may be removed
	  put_file_to_out -n "${DIR_ID[i]}/$top_before_nav" top3

	  # -n may be removed
	  put_file_to_out -n "${DIR_ID[i]}/$fnav"

	  # -n may be removed
	  put_file_to_out -n "${DIR_ID[i]}/$top_before_h" top4

	  # -n may be removed
	  echo -n "${DIM_HR[i]}"

	  put_file_to_out "${DIR_ID[i]}/$top_after_h" top5
  fi

  if [ -f "${DIR_ID[i]}"/$fnav4 ]; then
	  put_file_to_out "${DIR_ID[i]}/$top_before_nav4" top6

	  put_file_to_out "${DIR_ID[i]}"/$fnav4

	  put_file_to_out "${DIR_ID[i]}/$top_after_nav4" top7
  fi

  put_file_to_out "${DIR_ID[i]}/$top_before_content" top8

  put_file_to_out "$fcontent"

  put_file_to_out "${DIR_ID[i]}/$top_after_content" top9

  put_file_to_out "${DIR_ID[i]}/$footer" zfooter
  } > "$index_tmp"

  r=
  t="${DIR_ID[i]}/$prev_index_name"
  read_file_into_var r t no_error
  if [[ -z "$r" || $index != $r ]]; then
	[[ -n "$r" ]] && rm -f "$r"
	r=1
	echo "$index" > "${DIR_ID[i]}/$prev_index_name"
  else
	cmp "$index_tmp" "$index" > /dev/null 2>&1
	r=$?
  fi
  if [[ r -ne 0 ]]; then
	((u++))
	mv "$index_tmp" "$index" || exit 1
	uf=$index
	echo '[Updated]'
	[[ $one != "" ]] && break
  else
	((nu++))
	nuf=$index
	rm -f "$index_tmp"
	echo
  fi

done

d=0
if [[ max_old -gt 0 ]]; then
	for (( i=1; i<=menu_5_max; i++ )); do
		for (( i2=0; i2<max_old; i2++ )); do
			if [[ "${OLD_DIM_LINK[i2]}" = "${DIM_LINK[i]}" ]]; then
				OLD_DIM_LINK[i2]=
				break
			fi
		done
	done
	for (( i2=max_old-1; i2>=0; i2-- )); do
		if [[ -n "${OLD_DIM_LINK[i2]}" ]]; then
			if [[ d -eq 0 ]]; then
				echo
			fi
			((d++))
			rm -rf "..${OLD_DIM_LINK[i2]}"
			echo "..${OLD_DIM_LINK[i2]} [Deleted]"
		fi
	done
fi

if [[ -z $one ]]; then
  echo
  if [[ u -eq 1 ]]; then
	echo Updated 1 "file: $uf"
  else
	echo Updated "$u" files
  fi
  if [[ nu -eq 1 ]]; then
	echo Unchanged 1 "file: $nuf"
  else
	echo Unchanged "$nu" files
  fi

  if [[ nc -eq 1 ]]; then
	echo -n 1 directory with default "content: "
	cat "$out_dirs_default_content.$$"
  else
	echo -n $nc directories with default "content. "
  fi
  if [[ nc -gt 0 ]]; then
	cmp "$out_dirs_default_content.$$" "$out_dirs_default_content" > /dev/null 2>&1
	r=$?
	if [[ r -eq 0 ]]; then
		echo "$out_dirs_default_content" unchanged with previous call
		rm -f "$out_dirs_default_content.$$"
	else
		mv "$out_dirs_default_content.$$" "$out_dirs_default_content"
		echo See "$out_dirs_default_content"
	fi
  elif [ -f "$out_dirs_default_content" ]; then
	echo "$out_dirs_default_content" unchanged
  else
	echo
  fi
  if [[ d -eq 1 ]]; then
	echo Deleted 1 previous directory
  else
	echo Deleted $d previous directories
  fi
fi
# end -c
fi


if [[ $xD = D ]]; then
  for (( i=menu_5_max; i>=0; i-- )); do
    if [[ ext != ${DIM_TYPE[i]} ]]; then
	index=
	if [[ i -eq 0 ]]; then
		t="${DIR_ID[i]}/$prev_index_name"
		read_file_into_var index t no_error
	else
		if [ "${PREV_LEVEL[i]}" = 0 ]; then   # pl={"",0,1...}
			index=..${DIM_LINK[i]}
		elif [ -z "${PREV_LEVEL[i]}" ]; then
			echo rm -rf "..${DIM_LINK[i]}"
			rm -rf "..${DIM_LINK[i]}"  || exit 1
		fi
	fi
	if [ "$index" ]; then
		echo rm -f "$index"
		rm -f "$index" || exit 1
	fi
    fi
  done
  if [ "${DIM_COPY[0]}" ]; then
	pushd "${DIR_ID[0]}" > /dev/null
	FILES_LIST=(${DIM_COPY[0]})
	popd > /dev/null
	for l in ${FILES_LIST[@]}; do
		if [ -e "../$l" ]; then
			echo rm "../$l"
			rm "../$l"
		fi
	done
  fi

  if [ "${DIM_COPY2[0]}" ]; then
	FILES_LIST=(${DIM_COPY2[0]})
	for (( t=0; t<${#FILES_LIST[@]}; t++ )); do
		((t++))
		if [ ${#FILES_LIST[@]} -eq $t ]; then
			echo Error: "${DIR_ID[0]}/var:" COPY2 must have pairs 1>&2
			exit 1
		fi
		l="../${FILES_LIST[t]}"
		if [ -e "$l" ]; then
			echo rm "$l"
			rm "$l"
		fi
	done
  fi
fi

if [[ $xC = C ]]; then
	echo clear "$out $out_dirs_default_content $fmenus $ftitles $faccordions \
$fnav $fnav4 d_$prg *~"
	find . \( -name $out -o -name $out_dirs_default_content \
	-o -name $fmenus -o -name $ftitles -o -name d_$prg \
	-o -name $faccordions -o -name $fnav -o -name $fnav4 \
	-o -name '*~' -o -name $prev_index_name \) \
	-print0 | xargs -0 rm -f
fi
